// $Id: FuzzyXMLEnvironment.java,v 1.1.1.1 2006-03-22 13:32:17 pierre Exp $

package com.senellart.pierre.fuzzyxml;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import com.senellart.pierre.fuzzyxml.exception.FuzzyXMLConfigurationException;
import com.senellart.pierre.fuzzyxml.exception.FuzzyXMLUnsupportedException;

/**
 * @author pierre
 * 
 */
public class FuzzyXMLEnvironment {
	protected DocumentBuilderFactory domFactory;

	protected SAXParserFactory saxFactory;

	protected TransformerFactory xsltFactory;

	protected Transformer fuzzyxml2dot;

	protected Transformer query2dot;

	protected String path;

	protected File tempDir;

	protected DocumentBuilder domParser;

	protected class MyURIResolver implements URIResolver {
		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.xml.transform.URIResolver#resolve(java.lang.String,
		 *      java.lang.String)
		 */
		public Source resolve(String href, String base)
				throws TransformerException {
			if (!href.contains("://") && href.length() > 0
					&& !"/".equals(href.substring(0, 1)))
				return new StreamSource(path + "/" + href);
			else
				return new StreamSource(href);
		}
	}

	public static class MyErrorHandler implements ErrorHandler {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.xml.sax.ErrorHandler#warning(org.xml.sax.SAXParseException)
		 */
		public void warning(SAXParseException exception) {
			System.err.println(exception.getMessage());
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.xml.sax.ErrorHandler#error(org.xml.sax.SAXParseException)
		 */
		public void error(SAXParseException exception) throws SAXException {
			throw exception;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.xml.sax.ErrorHandler#fatalError(org.xml.sax.SAXParseException)
		 */
		public void fatalError(SAXParseException exception) throws SAXException {
			throw exception;
		}

	}

	public FuzzyXMLEnvironment(String path)
			throws FuzzyXMLConfigurationException {
		this(path, null);
	}

	/**
	 * @throws FuzzyXMLConfigurationException
	 */
	public FuzzyXMLEnvironment(String path, String tempDir)
			throws FuzzyXMLConfigurationException {
		if (tempDir == null)
			this.tempDir = null;
		else
			this.tempDir = new File(tempDir);

		this.path = path;

		URIResolver uriResolver = new MyURIResolver();

		if (System.getProperty("javax.xml.parsers.SAXParserFactory") == null)
			System.setProperty("javax.xml.parsers.SAXParserFactory",
					"org.apache.xerces.jaxp.SAXParserFactoryImpl");

		if (System.getProperty("javax.xml.parsers.DocumentBuilderFactory") == null)
			System.setProperty("javax.xml.parsers.DocumentBuilderFactory",
					"org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");

		if (System.getProperty("javax.xml.transform.TransformerFactory") == null)
			System.setProperty("javax.xml.transform.TransformerFactory",
					"org.apache.xalan.processor.TransformerFactoryImpl");

		domFactory = DocumentBuilderFactory.newInstance();
		domFactory.setNamespaceAware(true);
		domFactory.setValidating(false);

		domFactory.setAttribute(
				"http://java.sun.com/xml/jaxp/properties/schemaLanguage",
				"http://www.w3.org/2001/XMLSchema");

		try {
			domParser = domFactory.newDocumentBuilder();
			domParser.setErrorHandler(new MyErrorHandler());
		} catch (ParserConfigurationException e1) {
			// Should not happen
			e1.printStackTrace();
			System.exit(1);
		}

		saxFactory = SAXParserFactory.newInstance();
		saxFactory.setNamespaceAware(true);
		saxFactory.setValidating(false);

		xsltFactory = TransformerFactory.newInstance();
		xsltFactory.setURIResolver(uriResolver);

		try {
			fuzzyxml2dot = xsltFactory.newTransformer(new StreamSource(
					new FileInputStream(path + "/xsl/fuzzyxml2dot.xsl")));
			query2dot = xsltFactory.newTransformer(new StreamSource(
					new FileInputStream(path + "/xsl/query2dot.xsl")));
		} catch (Exception e) {
			// Should not happen
			throw new FuzzyXMLUnsupportedException(e);
		}
	}

	/**
	 * @return Returns the domFactory.
	 */
	public DocumentBuilderFactory getDOMFactory() {
		return domFactory;
	}

	/**
	 * @return Returns the xsltFactory.
	 */
	public TransformerFactory getXSLTFactory() {
		return xsltFactory;
	}

	public void serialize(Document doc, OutputStream out)
			throws TransformerException {
		OutputFormat outFormat = new OutputFormat(doc);
		XMLSerializer serializer = new XMLSerializer(outFormat);
		serializer.setNamespaces(true);
		serializer.setOutputByteStream(out);
		try {
			serializer.serialize(doc);
		} catch (IOException e) {
			// Should not happen
			throw new FuzzyXMLUnsupportedException(e);
		}
	}

	public void serialize(Document doc, Writer writer)
			throws TransformerException {
		OutputFormat outFormat = new OutputFormat(doc);

		XMLSerializer serializer = new XMLSerializer(outFormat);
		serializer.setNamespaces(true);
		serializer.setOutputCharStream(writer);
		try {
			serializer.serialize(doc);
		} catch (IOException e) {
			// Should not happen
			throw new FuzzyXMLUnsupportedException(e);
		}
	}

	public void fuzzyxml2dot(Document doc, OutputStream out,
			boolean showConfidence) throws TransformerException {
		fuzzyxml2dot.setParameter("with-confidence",new Boolean(showConfidence));
		fuzzyxml2dot.transform(new DOMSource(doc), new StreamResult(out));
	}

	public void fuzzyxml2dot(Document doc, OutputStream out)
			throws TransformerException {
		fuzzyxml2dot(doc, out, false);
	}

	public void query2dot(Document doc, OutputStream out)
			throws TransformerException {
		query2dot.transform(new DOMSource(doc), new StreamResult(out));
	}

	/**
	 * @return Returns the path.
	 */
	public String getPath() {
		return path;
	}

	/**
	 * @return Returns the domParser.
	 */
	public DocumentBuilder getDOMParser() {
		return domParser;
	}

	public SAXParserFactory getSAXFactory() {
		return saxFactory;
	}

	public File getTempDir() {
		return tempDir;
	}
}
