// $Id: FuzzyXMLServlet.java,v 1.1.1.1 2006-03-22 13:32:17 pierre Exp $

package com.senellart.pierre.fuzzyxml.demo;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;

import com.senellart.pierre.fuzzyxml.FuzzyXMLEnvironment;
import com.senellart.pierre.fuzzyxml.document.DocumentManager;
import com.senellart.pierre.fuzzyxml.document.FileDocumentManager;
import com.senellart.pierre.fuzzyxml.exception.FuzzyXMLUnsupportedException;
import com.senellart.pierre.util.GraphViz;

public abstract class FuzzyXMLServlet extends HttpServlet {
	protected FuzzyXMLEnvironment environment;

	protected DocumentManager documentManager;

	public void init() throws ServletException {
		try {
			environment = new FuzzyXMLEnvironment("/home/pierre/fuzzyxml");

			documentManager = new FileDocumentManager(environment);
		} catch (Exception e) {
			throw new ServletException(e);
		}
	}

	protected void doGet(HttpServletRequest req, HttpServletResponse res)
			throws ServletException, IOException {
		res.setContentType("text/xml");

		if ("HEAD".equals(req.getMethod()))
			return;
	}

	protected File createDotFile() throws IOException {
		File dotFile = File.createTempFile("fxml", ".dot", new File("/tmp/"));
		return dotFile;
	}

	protected void returnImage(HttpServletResponse res, File dotFile)
			throws IOException {
		File imgFile = new GraphViz(dotFile).generate("png", new File(
				environment.getPath() + "/img"));

		dotFile.delete();

		PrintWriter out = res.getWriter();
		out.print("<name>img/" + imgFile.getName() + "</name>");
	}

	protected void returnError(HttpServletResponse res, Exception exception)
		throws IOException {
		Document d = environment.getDOMParser().getDOMImplementation()
				.createDocument(null, "error", null);
		String message = exception.getMessage();
		if(message==null)
			message=exception.toString();
		int index = message.indexOf(": ");
		if (index == -1)
			index = -2;
		d.getDocumentElement().appendChild(
				d.createTextNode(message.substring(index + 2)));
		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		try {
			environment.serialize(d, stream);
		} catch(TransformerException e) { 
			throw new FuzzyXMLUnsupportedException(e);
		}
		
		res.getWriter().print(stream);
	}
}
