// $Id: EventFormula.java,v 1.2 2007-08-24 12:39:51 pierre Exp $

package com.senellart.pierre.fuzzyxml.event;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.senellart.pierre.fuzzyxml.exception.FuzzyXMLUnsupportedException;

abstract public class EventFormula {
	protected static class Assignment {
		Map valuations;

		Assignment() {
			valuations = new HashMap();
		}

		public void put(String id, boolean pos) {
			valuations.put(id, new Boolean(pos));
		}

		public Assignment copy() {
			Assignment a = new Assignment();

			for (Iterator it = getIds().iterator(); it.hasNext();) {
				String id = (String) it.next();
				a.put(id, ((Boolean) valuations.get(id)).booleanValue());
			}

			return a;
		}

		public Set getIds() {
			return valuations.keySet();
		}

		public boolean get(String id) {
			return ((Boolean) valuations.get(id)).booleanValue();
		}
	}

	public EventDisjunctiveFormula getDNF() {
		// Note: the formula returned is not minimal
		// This is quite tricky actually to find a minimal formula...
		// (w1,¬w2),(¬w1,w2),(w1,w2) -> (w1),(¬w1,w2)
		// Only one term should match, not more, so we do not want a full
		// minimization like (w1,¬w2),(¬w1,w2) -> (w1),(w2)
		EventDisjunctiveFormula f = new EventDisjunctiveFormula();
		Set ids = getIds();
		int nbIds = ids.size();
		String indexedIds[] = new String[nbIds];

		if (nbIds == 0)
			throw new FuzzyXMLUnsupportedException(
					"Error in call to getDNF()");

		int i = 0;
		for (Iterator it = ids.iterator(); it.hasNext();)
			indexedIds[i++] = (String) it.next();
		Set assignments = getAllAssignments(indexedIds, nbIds);

		for (Iterator it = assignments.iterator(); it.hasNext();) {
			Assignment a = (Assignment) it.next();
			if (evaluate(a))
				f.addTerm(getConjunctionFromAssignment(a));
		}

		return f;
	}

	abstract protected boolean evaluate(Assignment a);

	private static EventConjunction getConjunctionFromAssignment(Assignment a) {
		EventConjunction conj = new EventConjunction();

		for (Iterator it = a.getIds().iterator(); it.hasNext();) {
			String id = (String) it.next();
			conj.add(id, a.get(id));
		}

		return conj;
	}

	private static Set getAllAssignments(String indexedIds[], int nbIds) {
		Set assignments = new HashSet();

		String id = indexedIds[nbIds - 1];

		if (nbIds == 1) {
			Assignment a1 = new Assignment();
			a1.put(id, true);

			Assignment a2 = new Assignment();
			a2.put(id, false);
			assignments.add(a1);
			assignments.add(a2);
		} else {
			Set recAssignments = getAllAssignments(indexedIds, nbIds - 1);

			for (Iterator it = recAssignments.iterator(); it.hasNext();) {
				Assignment a1 = (Assignment) it.next();
				Assignment a2 = (Assignment) a1.copy();
				a1.put(id, true);
				a2.put(id, false);
				assignments.add(a1);
				assignments.add(a2);
			}
		}

		return assignments;
	}

	abstract public Set getIds();
}
